// Output generated from file: D:/tracers2/Mandelbulb_kaliset.frag
// Created:  13.  14:34:34 2015
#info Mandelbulb Distance Estimator
#define providesInit
#include "MathUtils.frag"
#include "DE-kn9.frag"
//#include "Volumetric.frag"


#group Mandelbulb

// Number of fractal iterations.
uniform int Iterations;  slider[0,9,100]

// Number of color iterations.
uniform int ColorIterations;  slider[0,9,100]

// Mandelbulb exponent (8 is standard)
uniform float Power; slider[0,8,16]

// Bailout radius
uniform float Bailout; slider[0,5,30]

// Alternate is slightly different, but looks more like a Mandelbrot for Power=2
uniform bool AlternateVersion; checkbox[false]

uniform vec3 RotVector; slider[(0,0,0),(1,1,1),(1,1,1)]

uniform float RotAngle; slider[0.00,0,180]

uniform bool Julia; checkbox[false]
uniform vec3 JuliaC; slider[(-2,-2,-2),(0,0,0),(2,2,2)]

mat3 rot;
uniform float time;
void init() {
	 rot = rotationMatrix3(normalize(RotVector), RotAngle);
       float dummy = time*10.0;
}

// This is my power function, based on the standard spherical coordinates as defined here:
// http://en.wikipedia.org/wiki/Spherical_coordinate_system
//
// It seems to be similar to the one Quilez uses:
// http://www.iquilezles.org/www/articles/mandelbulb/mandelbulb.htm
//
// Notice the north and south poles are different here.
void powN1(inout vec3 z, float r, inout float dr) {
	// extract polar coordinates
	float theta = acos(z.z/r);
	float phi = atan(z.y,z.x);
	dr =  pow( r, Power-1.0)*Power*dr + 1.0;
	
	// scale and rotate the point
	float zr = pow( r,Power);
	theta = theta*Power;
	phi = phi*Power;
	
	// convert back to cartesian coordinates
	z = zr*vec3(sin(theta)*cos(phi), sin(phi)*sin(theta), cos(theta));
}

// This is a power function taken from the implementation by Enforcer:
// http://www.fractalforums.com/mandelbulb-implementation/realtime-renderingoptimisations/
//
// I cannot follow its derivation from spherical coordinates,
// but it does give a nice mandelbrot like object for Power=2
void powN2(inout vec3 z, float zr0, inout float dr) {
	float zo0 = asin( z.z/zr0 );
	float zi0 = atan( z.y,z.x );
	float zr = pow( zr0, Power-1.0 );
	float zo = zo0 * Power;
	float zi = zi0 * Power;
	dr = zr*dr*Power + 1.0;
	zr *= zr0;
	z  = zr*vec3( cos(zo)*cos(zi), cos(zo)*sin(zi), sin(zo) );
}



// Compute the distance from `pos` to the Mandelbox.
float DE1(vec3 pos) {
	vec3 z=pos;
	float r;
	float dr=1.0;
	int i=0;
	r=length(z);
	while(r<Bailout && (i<Iterations)) {
		if (AlternateVersion) {
			powN2(z,r,dr);
		} else {
			powN1(z,r,dr);
		}
		z+=(Julia ? JuliaC : pos);
		r=length(z);
		z*=rot;
		//if (i<ColorIterations) orbitTrap = min(orbitTrap, abs(vec4(z.x,z.y,z.z,r*r)));
		i++;
	}
//	if ((type==1) && r<Bailout) return 0.0;
	return 0.5*log(r)*r/dr;
	/*
	Use this code for some nice intersections (Power=2)
	float a =  max(0.5*log(r)*r/dr, abs(pos.y));
	float b = 1000;
	if (pos.y>0)  b = 0.5*log(r)*r/dr;
	return min(min(a, b),
		max(0.5*log(r)*r/dr, abs(pos.z)));
	*/
}

float surf(vec3 pos) {

float k = Kaliset(pos);

float KDist = 0.0;
vec3 kdir=normalize(K_Dir);


if (KTex) {orbitTrap=vec4(1,1,1,1)*(k*ColorScale+ColorOffset);}
if (KHeightmap) {float sc=k*Strength; pos+=vec3(sc,sc,sc)*Strength2;}

KDist = abs(dot(kdir,pos)-kHeight);


return min(KDist,1);
}

float DE(vec3 pos) {
 return min(DE1(pos), surf(pos));
//return D1(pos);
}















#preset default
FOV = 0.46552
UseFocalLength = true
FocalLength = 47.318
FilmGate = 63.2679
Eye = 1.81997,-0.0263586,-2.11349
Ey_ = 0,0,0
Target = -0.498762,4.22654,5.30705
Targe_ = 0,0,0
FocalPlane = 1
Aperture = 0
InFocusAWidth = 0
DofCorrect = true
ApertureNbrSides = 5
ApertureRot = 0
ApStarShaped = false
Gamma = 1.2019
ToneMapping = 5
Exposure = 0.6522
Brightness = 1
Contrast = 2
Saturation = 1
GaussianWeight = 1.5
AntiAliasScale = 2
Bloom = true
BloomIntensity = 0.25
BloomPow = 5.3488
BloomTaps = 20
KTex = true
KIterations = 33
KScale = 2.1111
KZoom = 3
KFold = 0.24779,0.74336,0.25664
KJulia = -0.47872,-0.5,-0.27103
KPosition = 0,0.4652,-1.8604
KBailout = 0
KFinalScale = 0.11111
KFinalPow = 2.33765
KRotVector = 1,0,0
KRotAngle = 7.0956
KExpSmoothing = true
ColorScale = 0.35955
ColorOffset = 1
KHeightmap = true
Strength = 0.0845
Strength2 = 0,0.45204,3
K_Dir = -0.115,2.4444,-0.3448
kHeight = 0.47573
Detail = -3.2
DetailAO = -1.25265
FudgeFactor = 0.74359
MaxRaySteps = 841
MaxDistance = 10
Dither = 1
NormalBackStep = 1
AO = 0,0,0,1
AoCorrect = 0
Specular = 1.46938
SpecularExp = 500
CamLight = 0.843137,0.921569,1,0.5
CamLightMin = 1
Glow = 1,1,1,0.42353
GlowMax = 52
Reflection = 0.196078,0.32549,0.403922
ReflectionsNumber = 2
SpotGlow = true
SpotLight = 1,0.745098,0.627451,2.23075
LightPos = 3.3334,2,6.4444
LightSize = 0.33673
LightFallOff = 0
LightGlowRad = 1
LightGlowExp = 0.85525
HardShadow = 1 Locked
ShadowSoft = 0
ShadowBlur = 0.14286
perf = false
BaseColor = 1,1,1
OrbitStrength = 1
X = 0.5,0.6,0.6,0.65656
Y = 1,0.6,0,-0.2
Z = 0.8,0.78,1,1
R = 0.4,0.7,1,0.18368
BackgroundColor = 0.419608,0.74902,1
GradientBackground = 0.5814
CycleColors = false
Cycles = 0.1
EnableFloor = false
FloorNormal = 0.14286,1,-0.11688
FloorHeight = 0.0633
FloorColor = 1,1,1
HF_Fallof = 1
HF_Const = 0
HF_Intensity = 0.021
HF_Dir = 0,1,0
HF_Offset = 2.5
HF_Color = 1,0.807843,0.741176,0.90909
GodRays = true
HF_Scatter = 6.7089
HF_Anisotropy = 0.145098,0.203922,0.25098
HF_FogIter = 16
HF_CastShadow = false
ElableClouds = true
EnCloudsDir = true
Clouds_Dir = 0.14286,0.11904,-1
CloudScale = 0.95556
CloudFlatness = 3.492
CloudTops = 0.44
CloudBase = -4
CloudDensity = 0.21053
CloudRoughness = 0.23728
CloudContrast = 14.4
CloudColor = 0.733333,0.756863,0.811765
SunLightColor = 1,0.701961,0.627451
Iterations = 4
ColorIterations = 14
Power = 2
Bailout = 2
AlternateVersion = false
RotVector = 1,0,0
RotAngle = 0
Julia = true
JuliaC = -0.39252,1,2
Up = 0.0385936,0.66492,-0.369023
#endpreset

